function[Satisfied, Edges]=QuenchedLS2(Lambdamu,Lambdasig,Links,Importance,Zeta)

%Predetermined Values
Small=0.000000001;
Infi=10000000000;
eqTime=100;
eqCount=0;
NoOfSteps=3000;
%Rose=0.01; %% 1/level of importance


N=length(Lambdamu); %Number of nodes in the network

%Writing the Links vector as a Matrix of connections
[NeighbourM,NeighbourCM,NeighbourICon]=MakingNeighbourMatrixCM(Links,N);

%Starting values of A, B and Y
Max=max(NeighbourICon);
A=zeros(N,Max);
B=ones(N,Max);
Y=zeros(N,Max);

%Exhaustively and randomly choosing all trees for iterations
LLinks=[Links(1,:) Links(2,:); Links(2,:) Links(1,:)];

for j=0:length(LLinks):(NoOfSteps*N)
    for i=1:length(LLinks)
        a=randi(length(LLinks)-i+1);
        Randi(:,i+j)=LLinks(:,a);
        b=LLinks(:,a);
        LLinks(:,a)=[];
        LLinks=[LLinks b];
    end
end

for t=1:NoOfSteps*N
   
    %Assigning Nodes J and I
    nodeJ=Randi(1,t);
    connectivity=NeighbourICon(nodeJ);
    nodeI=Randi(2,t);
    cavityI=find(NeighbourM(nodeJ,:)==nodeI);
    cavityJ=find(NeighbourM(nodeI,:)==nodeJ);
    
    Totali=sum(Lambdamu)-Lambdamu(nodeJ);
        
    %Rewriting for ease
    Yij=Y(nodeI,cavityJ);
    Yji=Y(nodeJ,cavityI);
    Aij=A(nodeI,cavityJ);
    Aji=A(nodeJ,cavityI);
    Bij=B(nodeI,cavityJ);
    Bji=B(nodeJ,cavityI);
    mu=Lambdamu(nodeJ);
    sig=Lambdasig(nodeJ);

    %Assuming the quadratic transportation cost
    Phiij1=Y(nodeI,cavityJ); %*
    Phiij2=1; %*
    
    %Obtaining values of A, B and Y from descendants k
    for k=1:connectivity
        Ajk(k)=A(nodeJ,k);
        Bjk(k)=B(nodeJ,k);
        Yjk(k)=Y(nodeJ,k);
        Phijk1(k)=Y(nodeJ,k); %*
        Phijk2(k)=1; %*
    end
    
    %Summing the descendants for calculations in mu
    tempd1=0;
    tempd2=0;
    
    for k=1:connectivity
        if k~=cavityI
            tempd1=tempd1+Yjk(k)-((Ajk(k)+Phijk1(k))/(Bjk(k)+Phijk2(k)));
            tempd2=tempd2+(1/(Phijk2(k)+Bjk(k)));
        end
    end
    
    %Calculating Mu
    x=Yij-tempd1;
    S2=sqrt(2);
    SP=sqrt(pi);
    mum=mu+(S2*sig*erfinv((2*Zeta)-1));
    z=(x-mum)/(S2*sig);
    ERF=erf(z);
    EXP=exp(-(z^2));
    zi=(-Totali-mum)/(S2*sig);
    ERFI=erf(zi);
    EXPI=exp(-(zi^2));
    
    if sig==0
        if x-mum>0
            ERF=1;
        elseif x-mum<0
            ERF=-1;
        else
            ERF=0;
        end
        EXP=0;
        if -Totali-mum>0
            ERFI=1;
        elseif -Totali-mum<0
            ERFI=-1;
        else
            ERFI=0;
        end
        EXPI=0;
    end    
    
    if x<=-Totali
        tempmu=(((mum-x)*(ERF+1))-((S2*sig*EXP)/(SP)))/(2*(tempd2+(1/Importance(nodeJ))));
    else
        tempmu=(((mum-x)*(ERFI+1))-((S2*sig*EXPI)/(SP)))/(2*(tempd2+(1/Importance(nodeJ))))+(((mum-x)*(ERF-ERFI))-((S2*sig*EXP)/(SP))+((S2*sig*EXPI)/(SP)))/(2*(tempd2));
    end
    
    %Messages before changing to check convergence
    preA=Aij;
    preB=Bij;
    preY=Yij;

    %Calculating new Aij and Bij
    A(nodeI,cavityJ)=-tempmu;
    
    if x<-Totali
        B(nodeI,cavityJ)=(ERF+1)/(2*(tempd2+(1/Importance(nodeJ))));
    else
        B(nodeI,cavityJ)=(ERFI+1)/(2*(tempd2+(1/Importance(nodeJ))))+(ERF-ERFI)/(2*(tempd2));
    end
    
    
    Aij=A(nodeI,cavityJ); %ease
    Bij=B(nodeI,cavityJ); %ease
    
    %Calculating new Yij
    Y(nodeI,cavityJ)=(((Bij*Yij)-Aij-(Bji*Yji)+Aji-Phiij1+(Phiij2*Yij))/(Bij+Bji+Phiij2));

    %ConvergenceCheck
    fieldChange=abs(A(nodeI,cavityJ)-preA)+abs(B(nodeI,cavityJ)-preB)+abs(Y(nodeI,cavityJ)-preY);

    if fieldChange<Small
        eqCount=eqCount+1;
    else
        eqCount=0;
    end
    
    %Finish if converged.
    if eqCount>eqTime*N
        fprintf('Break at %5.0f Iterations for N=%3.0f \n',t,N);
        break;
    end
    
end

[Satisfied,TotalSat,TotalLam]=Check(Y,Lambdamu,NeighbourICon,Small);
[Edges]=BackYAsEdgesEdges(Links,Y,NeighbourM,Small,NeighbourCM);

if abs(TotalLam-TotalSat)>0.01 || any(abs(Edges(:,3)+Edges(:,4))>0.01)
    disp('This didn''t work')
    TotalLam
    TotalSat
    Edges
end

end

%* Change these is quadratic transportation cost changes.